﻿using System;
using Un4seen.Bass;
using Windows.ApplicationModel.Background;
using Windows.Foundation.Collections;
using Windows.Media.Core;
using Windows.Media.Playback;

namespace BackgroundPlayer
{
  public sealed class Player : IBackgroundTask
  {
    BackgroundTaskDeferral m_Deferral;
    BassMediaSource m_MediaSourceAdapter = null;

    public void Run(IBackgroundTaskInstance taskInstance)
    {
      m_Deferral = taskInstance.GetDeferral();
      taskInstance.Canceled += TaskInstance_Canceled;
      taskInstance.Task.Completed += TaskInstance_Completed;

      BassNet.Registration("Your email", "Your registration key");
      if (!Bass.BASS_Init(0, 44100, BASSInit.BASS_DEVICE_DEFAULT, IntPtr.Zero)) {
        BASSError err = Bass.BASS_ErrorGetCode();
        m_Deferral.Complete();
        return;
      }

      BackgroundMediaPlayer.MessageReceivedFromForeground += OnMessageReceived;
    }

    private void TaskInstance_Canceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason reason)
    {
      Bass.BASS_Free();
      m_Deferral.Complete();
    }

    void TaskInstance_Completed(BackgroundTaskRegistration sender, BackgroundTaskCompletedEventArgs args)
    {
      Bass.BASS_Free();
      m_Deferral.Complete();
    }

    private async void OnMessageReceived(object sender, MediaPlayerDataReceivedEventArgs args)
    {
      ValueSet msg = args.Data;

      if (msg.ContainsKey("file")) {
        string audioFile = msg["file"] as string;
        if (!string.IsNullOrEmpty(audioFile)) {
          m_MediaSourceAdapter = await BassMediaSource.CreateAsync(audioFile);
          BackgroundMediaPlayer.Current.Source = MediaSource.CreateFromIMediaSource(m_MediaSourceAdapter.GetMediaSource());
          BackgroundMediaPlayer.Current.Play();
        }
      }
    }
  }
}