﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Storage.Pickers;
using Windows.Storage;
using Windows.Media.Playback;

namespace BassUWPTest
{
  public sealed partial class MainPage : Page
  {
    public MainPage()
    {
      this.InitializeComponent();

      m_ApplicationFolder.Text = ApplicationData.Current.LocalFolder.Path;

      BackgroundMediaPlayer.Current.CurrentStateChanged += Current_CurrentStateChanged;

      DispatcherTimer timer = new DispatcherTimer();
      timer.Interval = TimeSpan.FromMilliseconds(500);
      timer.Tick += Timer_Tick;
      timer.Start();
    }

    private async void Current_CurrentStateChanged(MediaPlayer sender, object args)
    {
      switch (sender.PlaybackSession.PlaybackState) {
        case MediaPlaybackState.Playing:
          await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Low, () => {
            m_PlayPause.Content = "Pause";
            m_PlayPause.IsEnabled = true;
          });          
          break;
        case MediaPlaybackState.Paused:
          await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Low, () =>
          {
            m_PlayPause.Content = "Play";
            m_PlayPause.IsEnabled = true;
          });
          break;
        case MediaPlaybackState.None:
        case MediaPlaybackState.Opening:
          await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Low, () =>
          {
            m_PlayPause.IsEnabled = false;
          });
          break;
      }
    }

    private void Timer_Tick(object sender, object e)
    {
      if (BackgroundMediaPlayer.Current.PlaybackSession.PlaybackState != MediaPlaybackState.None) {
        m_Progress.Maximum = BackgroundMediaPlayer.Current.PlaybackSession.NaturalDuration.TotalMilliseconds;
        m_Progress.Value = BackgroundMediaPlayer.Current.PlaybackSession.Position.TotalMilliseconds;

        m_Time.Text = string.Format("{0}/{1}", BackgroundMediaPlayer.Current.PlaybackSession.NaturalDuration, BackgroundMediaPlayer.Current.PlaybackSession.Position);
      }
    }

    private async void Button_Click(object sender, RoutedEventArgs e)
    {
      FileOpenPicker opener = new FileOpenPicker();
      opener.ViewMode = PickerViewMode.Thumbnail;
      opener.SuggestedStartLocation = PickerLocationId.MusicLibrary;
      opener.FileTypeFilter.Add(".mp3");
      opener.FileTypeFilter.Add(".ogg");
      opener.FileTypeFilter.Add(".flac");

      StorageFile file = await opener.PickSingleFileAsync();
      if (file != null)
        SendMessageToBackground("file", file.Path);
    }

    private void SendMessageToBackground(string message, object args)
    {
      try {
        BackgroundMediaPlayer.SendMessageToBackground(new ValueSet() { { message, args } });
      } catch (Exception ex) {
        System.Diagnostics.Debug.WriteLine(string.Format("BackgroundMediaPlayer error: {0}\r\n{1}", message, ex.Message));
      }
    }

    private void m_PlayPause_Click(object sender, RoutedEventArgs e)
    {
      if (BackgroundMediaPlayer.Current.PlaybackSession.PlaybackState == MediaPlaybackState.Playing)
        BackgroundMediaPlayer.Current.Pause();
      else if (BackgroundMediaPlayer.Current.PlaybackSession.PlaybackState == MediaPlaybackState.Paused)
        BackgroundMediaPlayer.Current.Play();        
    }
  }
}
